@@ -38,11 +38,14 @@ enum _MuMsgFieldId {
 	MU_MSG_FIELD_ID_MAILDIR,
 	MU_MSG_FIELD_ID_MSGID,
 	MU_MSG_FIELD_ID_PATH,
-	MU_MSG_FIELD_ID_REFS,
 	MU_MSG_FIELD_ID_SUBJECT,
 	MU_MSG_FIELD_ID_TO,
 	/* MU_MSG_STRING_FIELD_ID_NUM, see below */
 
+	/* string list items... */
+	MU_MSG_FIELD_ID_REFS,
+	MU_MSG_FIELD_ID_TAGS,
+	
 	/* then the numerical ones */
 	MU_MSG_FIELD_ID_DATE,
 	MU_MSG_FIELD_ID_FLAGS,
@@ -51,33 +54,28 @@ enum _MuMsgFieldId {
 	MU_MSG_FIELD_ID_TIMESTAMP,
 	
 	MU_MSG_FIELD_ID_NUM,
-
-	/* fake fields */
-	MU_MSG_PSEUDO_FIELD_ID_DATESTR,  /* used in to store datestr in db */
-	MU_MSG_FIELD_ID_NUM_WITH_PSEUDO /* number including these fake
-					 * fields */ 
 };
-typedef enum _MuMsgFieldId MuMsgFieldId;
+typedef guint8 MuMsgFieldId;
 
 /* some specials... */
 static const MuMsgFieldId MU_MSG_FIELD_ID_NONE = (MuMsgFieldId)-1;
 #define MU_MSG_STRING_FIELD_ID_NUM (MU_MSG_FIELD_ID_TO + 1)
 
-
 #define mu_msg_field_id_is_valid(MFID) \
 	((MFID) < MU_MSG_FIELD_ID_NUM)
 
 /* don't change the order, add new types at the end (before _NUM)*/
 enum _MuMsgFieldType {
 	MU_MSG_FIELD_TYPE_STRING,
-
+	MU_MSG_FIELD_TYPE_STRING_LIST,
+	
 	MU_MSG_FIELD_TYPE_BYTESIZE, 
 	MU_MSG_FIELD_TYPE_TIME_T,
 	MU_MSG_FIELD_TYPE_INT,
 
 	MU_MSG_FIELD_TYPE_NUM
 };
-typedef enum _MuMsgFieldType MuMsgFieldType;
+typedef guint8 MuMsgFieldType;
 static const MuMsgFieldType MU_MSG_FIELD_TYPE_NONE = (MuMsgFieldType)-1;
 
 typedef void (*MuMsgFieldForEachFunc) (MuMsgFieldId id,
@@ -145,6 +143,18 @@ MuMsgFieldType mu_msg_field_type (MuMsgFieldId id) G_GNUC_PURE;
 #define mu_msg_field_is_string(MFID)\
 	(mu_msg_field_type((MFID))==MU_MSG_FIELD_TYPE_STRING?TRUE:FALSE)
 
+
+
+/**
+ * is the field a string-list?
+ * 
+ * @param id a MuMsgFieldId
+ * 
+ * @return TRUE if the field a string-list, FALSE otherwise
+ */
+#define mu_msg_field_is_string_list(MFID)\
+	(mu_msg_field_type((MFID))==MU_MSG_FIELD_TYPE_STRING_LIST?TRUE:FALSE)
+
 /**
  * is the field numeric (has type MU_MSG_FIELD_TYPE_(BYTESIZE|TIME_T|INT))?
  * 
@@ -235,7 +245,7 @@ gboolean mu_msg_field_xapian_escape (MuMsgFieldId id) G_GNUC_PURE;
 
 /**
  * should this field be normalized? ie. should it be downcased and
- * accents removed?
+ * accents removed when storing as Xapian term?
  * 
  * @param field a MuMsgField
  * 
@@ -243,6 +253,7 @@ gboolean mu_msg_field_xapian_escape (MuMsgFieldId id) G_GNUC_PURE;
  */
 gboolean mu_msg_field_normalize (MuMsgFieldId id) G_GNUC_PURE;
 
+
 /**
  * should this field be stored as contact information? This means that
  * e-mail address will be stored as terms, and names will be indexed
