@@ -51,6 +51,8 @@
 #include <glib/gstdio.h>
 #include <errno.h>
 
+#include <langinfo.h>
+
 static char*
 do_wordexp (const char *path)
 {
@@ -82,6 +84,7 @@ do_wordexp (const char *path)
 }
 
 
+
 /* note, the g_debugs are commented out because this function may be
  * called before the log handler is installed. */
 char*
@@ -117,7 +120,9 @@ char*
 mu_util_create_tmpdir (void)
 {
 	gchar *dirname;
-        dirname =  g_strdup_printf ("%s%cmu-%d%c%x", g_get_tmp_dir(),
+	
+        dirname =  g_strdup_printf ("%s%cmu-%d%c%x",
+				    g_get_tmp_dir(),
 				    G_DIR_SEPARATOR,
 				    getuid(),
 				    G_DIR_SEPARATOR,
@@ -151,16 +156,20 @@ mu_util_init_system (void)
 	/* without setlocale, non-ascii cmdline params (like search
 	 * terms) won't work */
 	setlocale (LC_ALL, "");
-
+	
 	/* on FreeBSD, it seems g_slice_new and friends lead to
-	 * segfaults. So we shut if off */
-#ifdef 	__FreeBSD__
+	 * segfaults. Same for MacOS. We cannot easily debug what is
+	 * going on there (no access to such a system), so all we can
+	 * do is add a lame fallback -> we let g_slice_* use normal
+	 * malloc
+	 */
+#ifndef __linux__
 	if (!g_setenv ("G_SLICE", "always-malloc", TRUE)) {
 		g_critical ("cannot set G_SLICE");
 		return FALSE;
 	}
-	MU_WRITE_LOG("setting G_SLICE to always-malloc");
-#endif /*__FreeBSD__*/
+	/* g_debug ("setting G_SLICE to always-malloc"); */
+#endif /*!__linux__*/
 
 	g_type_init ();
 
@@ -370,3 +379,103 @@ mu_util_get_dtype_with_lstat (const char *path)
 }
 
 
+gboolean
+mu_util_locale_is_utf8 (void)
+{
+	const gchar *dummy;
+	static int is_utf8 = -1;
+	
+	if (G_UNLIKELY(is_utf8 == -1)) 
+	    	is_utf8 = g_get_charset(&dummy) ? 1 : 0;
+	
+	return is_utf8 ? TRUE : FALSE;
+}
+
+gboolean
+mu_util_fputs_encoded (const char *str, FILE *stream)
+{
+	char *conv;
+	int rv;
+	
+	g_return_val_if_fail (str, FALSE);
+	g_return_val_if_fail (stream, FALSE);
+	
+	/* g_get_charset return TRUE when the locale is UTF8 */
+	if (mu_util_locale_is_utf8()) 
+		rv = fputs (str, stream);
+	else { /* charset is _not_ utf8, so we actually have to
+		* convert it..*/
+		GError *err;
+		unsigned bytes;
+		err = NULL;
+	
+		conv = g_locale_from_utf8 (str, -1, &bytes, NULL, &err);
+		if (err) {
+			/* conversion failed; this happens because is
+			 * some cases GMime may gives us non-UTF-8
+			 * string from e.g. wrongly encoded
+			 * message-subjects; if so, we escape the
+			 * string */
+			g_free (conv);
+			conv = g_strescape (str, NULL);
+			g_error_free (err);
+			return FALSE;
+		}
+		rv = fputs (conv, stream);
+		g_free (conv);
+	}
+	
+	if (rv == EOF) { /* note, apparently, does not set errno */
+		g_printerr ("fputs failed");
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+
+static gboolean
+print_args (FILE *stream, const char *frm, va_list args)
+{
+	gchar *str;
+	gboolean rv;
+	
+	str = g_strdup_vprintf (frm, args);
+
+	rv = mu_util_fputs_encoded (str, stream);
+
+	g_free (str);
+
+	return rv;
+}
+
+
+gboolean
+mu_util_print_encoded (const char *frm, ...)
+{
+	va_list args;
+	gboolean rv;
+
+	g_return_val_if_fail (frm, FALSE);
+	
+	va_start (args, frm);
+	rv = print_args (stdout, frm, args);
+	va_end (args);
+
+	return rv;
+}
+
+gboolean
+mu_util_printerr_encoded (const char *frm, ...)
+{
+	va_list args;
+	gboolean rv;
+
+	g_return_val_if_fail (frm, FALSE);
+	
+	va_start (args, frm);
+	rv = print_args (stderr, frm, args);
+	va_end (args);
+
+	return rv;
+}
