@@ -29,7 +29,7 @@ main (int argc, char *argv[])
 {
 	int rv;
 	
-	if (!mu_runtime_init_from_cmdline (&argc, &argv))
+	if (!mu_runtime_init_from_cmdline (&argc, &argv, "mu"))
 		return 1;
 
 	rv = mu_config_execute (mu_runtime_config());
